@@ -59,10 +59,10 @@
 #' @param max_bundles Maximal number of bundles to get. Defaults to Inf meaning all available bundles are downloaded.
 #' @param verbose An integer vector of length one. If 0, nothing is printed, if 1, only finishing message is printed, if > 1,
 #' downloading progress will be printed. Defaults to 1.
-#' @param max_attempts `r lifecycle::badge("deprecated")` The number of maximal attempts is now determined by the length of `delay_between_attempts`
 #' @param delay_between_attempts A numeric vector specifying the delay in seconds between attempts of reaching the server
-#' that `fhir_search()` will make. The length of this vector determines the number of attempts that will be made before stopping with an error.
-#' Defaults to `c(1,3,9,27,81)`.
+#' that `fhir_search()` will make. The length of this vector determines the number of attempts that will be made when the server can't be reached
+#' before stopping with an error. Defaults to `c(1,3,9,27,81)`. If you want the function to stop immediately after the first error when trying to
+#' reach the server, set this argument to `1`.
 #' @param log_errors Either `NULL` or a character vector of length one indicating the name of a file in which to save the http errors.
 #' `NULL` means no error logging. When a file name is provided, the errors are saved in the specified file. Defaults to `NULL`.
 #' Regardless of the value of `log_errors` the most recent http error message within the current R session is saved internally and can
@@ -78,6 +78,11 @@
 #' @param rm_tag Character vector of length 1 defining an xml tag of elements that will removed from the bundle automatically.
 #' Defaults to `"div"`,leading to the removal of all html parts (see Details). Set to `NULL` to keep the bundles untouched.
 #' See [fhir_rm_div()] and [fhir_rm_tag()] for more info.
+#' @param stop_on_error Controls error handling for server requests
+#' - `stop_on_error = 0`: Issues warning and retries requests as defined by `delay_between_attempts`. Returns empty bundle_list if all attempts fail
+#' - `stop_on_error = 1`: Stops immediately on any error. Overrides `delay_between_attempts`, as no retries will be attempted.
+#' - `stop_on_error = <vector of HTTP codes>`: Stops on specified codes (e.g. c(401, 404)), warns for others
+#'
 #' @return A [fhir_bundle_list-class] when `save_to_disc = NULL` (the default),  else `NULL`.
 #' @export
 #'
@@ -137,15 +142,9 @@ fhir_search <- function(
 	save_to_disc           = NULL,
 	delay_between_bundles  = 0,
 	rm_tag                 = "div",
-	max_attempts           = deprecated()
-) {
-	if(lifecycle::is_present(max_attempts)) {
-		lifecycle::deprecate_warn(
-			when = "2.0.0",
-			what = "fhir_search(max_attempts)",
-			details = "The number of maximal attempts is now controlled by the length of the argument delay_between_attempts."
-		)
-	}
+	stop_on_error	       = 0
+	) {
+
 
 	if(is.null(request)) {
 		stop(
@@ -155,6 +154,8 @@ fhir_search <- function(
 		)
 	}
 
+	warn_div_removal <- FALSE
+
 	#Extract base URL
 	base <- stringr::str_match(request, ".*?:\\/\\/.*?\\/")
 
@@ -212,7 +213,7 @@ fhir_search <- function(
 	cnt <- 0
 	repeat {
 		cnt <- cnt + 1
-		if(1 < verbose) {message("bundle[", cnt, "]", appendLF = FALSE)}
+		if(1 < verbose) {message("bundle[", cnt, "]")}
 
 		bundle <- get_bundle(
 			request = addr,
@@ -222,16 +223,14 @@ fhir_search <- function(
 			token = token,
 			add_headers = add_headers,
 			verbose = verbose,
-			max_attempts = max_attempts,
 			delay_between_attempts = delay_between_attempts,
 			log_errors = log_errors,
-			rm_tag = rm_tag
+			rm_tag = rm_tag,
+			stop_on_error = stop_on_error
 		)
 
 		if(is.null(bundle)) {
-			if(0 < verbose) {
-				message("Download interrupted.")
-			}
+			message("Download interrupted.")
 			break
 		}
 
@@ -286,8 +285,11 @@ fhir_search <- function(
 	}
 
 	fhircrackr_env$current_request <- request
+	if(warn_div_removal){warning("In some of downloaded bundles the specified tag (default 'div') could not be removed",
+								 " to save memory. Set rm_tag = NULL to get rid of this warning."
+	)}
+	if(is.null(save_to_disc)) {fhir_bundle_list(bundles)} else {NULL}
 
-	if(is.null(save_to_disc)) {fhir_bundle_list(bundles)} else {NULL} #brauchts eigentlich auch nicht
 }
 
 
@@ -1003,7 +1005,6 @@ fhir_authenticate <- function(
 #' @param token The token for token based auth, either a string or a httr token object
 #' @param add_headers A named character vector of custom headers to add to the HTTP request, e.g. `c(myHeader = "somevalue")` or
 #' `c(firstHeader = "value1", secondHeader = "value2")`.
-#' @param max_attempts Deprecated. The number of maximal attempts is determined by the length of `delay_between_attempts`
 #' @param verbose An integer scalar. If > 1,  Downloading progress is printed. Defaults to 2.
 #' @param delay_between_attempts A numeric vector specifying the delay in seconds between attempts of reaching the server
 #' that `fhir_search()` will make. The length of this vector determines the number of attempts that will be made before stopping with an error.
@@ -1036,10 +1037,10 @@ get_bundle <- function(
 	token = NULL,
 	add_headers = NULL,
 	verbose = 2,
-	max_attempts = NULL,
 	delay_between_attempts = c(1,3,9,27,81),
 	log_errors = NULL,
-	rm_tag = "div") {
+	rm_tag = "div",
+	stop_on_error = 0) {
 
 	#download response
 	for(n in seq_along(delay_between_attempts)) {
@@ -1080,16 +1081,39 @@ get_bundle <- function(
 				auth$basicAuth
 			), silent = TRUE)
 		}
-		#check for errors
-		check_response(response = response, log_errors = log_errors)
+
+		#check for errors: Upgrade warning to error depending on stop_on_error
+		tryCatch(
+			expr = check_response(response = response, log_errors = log_errors),
+			warning = function(w){
+				if(any(stop_on_error==1)){
+					message("Stopping because stop_on_error = 1. Set stop_on_error=0 to continue with a warning instead.")
+					stop(w$message)
+				}else if(!is(response, "try-error") && response$status_code %in% stop_on_error){
+					message("Stopping because of HTTP code ", response$status_code,
+							".\nSet stop_on_error=0 to continue with a warning instead.")
+					stop(w$message)
+				}else{
+					warning(w)
+				}
+			})
 
 		#extract payload
 		payload <- try(httr::content(x = response, as = "text", encoding = "UTF-8"), silent = TRUE)
-		if(class(payload)[1] != "try-error") {
+		if(!inherits(payload, "try-error")) {
 			xml <- try(xml2::read_xml(x = payload), silent = TRUE)
-			if(class(xml)[1] != "try-error") {
+			if(!inherits(xml, "try-error")) {
 				bundle <- fhir_bundle_xml(bundle = xml)
-				if(!is.null(rm_tag)){bundle <- fhir_rm_tag(x = bundle, tag = rm_tag)}
+				#try to remove tag/div
+				if(!is.null(rm_tag)){
+					tag_removed_bundle <-try(fhir_rm_tag(x = bundle, tag = rm_tag), silent = TRUE)
+					if(!inherits(tag_removed_bundle, "try-error")){
+						bundle <- tag_removed_bundle
+					}else{
+						parent <- parent.frame()
+						parent$warn_div_removal <- TRUE
+					}
+				}
 				return(bundle)
 			}
 		}
@@ -1111,7 +1135,7 @@ error_to_file <- function(response, log_errors, append) {
 }
 #' Check http response
 #'
-#' Checks the http response and issues an error or warning if necessary
+#' Checks the http response and issues a warning if
 #'
 #' @param response A http response
 #' @param log_errors Either `NULL` or a character vector of length one indicating the name of a file in which to save the http errors.
@@ -1125,30 +1149,30 @@ check_response <- function(response, log_errors, append = FALSE) {
 	#Error in curl
 	if(is(response, "try-error")){
 		if(!is.null(log_errors)){
-			write(x = response, file = log_errors)
-			stop("The server could not be reached:\n", response, ".\n",
-				 "This has been logged in the generated error file.")
-		}else{
-			stop("The server could not be reached:\n", response, ".\n")
+			write(x = response, file = log_errors, append = TRUE)
+		}
+		warning("The server could not be reached:\n", response, "\n")
+	}
+
+	#http error: Any code >=400 is an error and needs to be logged
+	code <- response$status_code
+
+	if(code >= 400){
+		fhircrackr_env$recent_http_error <- httr::content(x = response, as = "text", encoding = "UTF-8")
+		if(!is.null(log_errors)){
+			error_to_file(response = response, log_errors = log_errors, append = append)
 		}
 	}
 
-	#http error
-	code <- response$status_code
-	if(code >= 400){
-		fhircrackr_env$recent_http_error <- httr::content(x = response, as = "text", encoding = "UTF-8")
-	}
-	if(code >= 400 && !is.null(log_errors)) {
-		error_to_file(response = response, log_errors = log_errors, append = append)
-	}
+	#give specific warnings for certain error types
 	if(code == 400) {
 		if (!is.null(log_errors)) {
-			stop(
+			warning(
 				"HTTP code 400 - This can be caused by an invalid request or a server issue. ",
 				"For more information see the generated error file."
 			)
 		} else {
-			stop(
+			warning(
 				"HTTP code 400 - This can be caused by an invalid request or a server issue. ",
 				"To print more detailed error information, run fhir_recent_http_error() or set argument log_errors to a filename and rerun you request."
 			)
@@ -1156,9 +1180,9 @@ check_response <- function(response, log_errors, append = FALSE) {
 	}
 	if(code == 401) {
 		if(!is.null(log_errors)) {
-			stop("HTTP code 401 - Authentication needed. For more information see the generated error file.")
+			warning("HTTP code 401 - Authentication needed. For more information see the generated error file.")
 		} else {
-			stop(
+			warning(
 				"HTTP code 401 - Authentication needed. To print more detailed error information,",
 				"run fhir_recent_http_error() or set argument log_errors to a filename and rerun you request."
 			)
@@ -1166,9 +1190,9 @@ check_response <- function(response, log_errors, append = FALSE) {
 	}
 	if(code == 404) {
 		if(!is.null(log_errors)) {
-			stop("HTTP code 404 - Not found. Did you misspell the resource? For more information see the generated error file.")
+			warning("HTTP code 404 - Not found. Did you misspell the resource? For more information see the generated error file.")
 		} else {
-			stop(
+			warning(
 				"HTTP code 404 - Not found. Did you misspell the resource? To print more detailed error information, ",
 				"run fhir_recent_http_error() or set argument log_errors to a filename and rerun you request."
 			)
@@ -1190,13 +1214,13 @@ check_response <- function(response, log_errors, append = FALSE) {
 	}
 	if(400 <= code && code < 500) {
 		if(!is.null(log_errors)) {
-			stop(
+			warning(
 				"Your request generated a client error, HTTP code ",
 				code,
 				". For more information see the generated error file."
 			)
 		} else {
-			stop(
+			warning(
 				"Your request generated a client error, HTTP code ",
 				code,
 				". To print more detailed error information, run fhir_recent_http_error() or set argument log_errors to a filename and rerun you request."
@@ -1205,13 +1229,13 @@ check_response <- function(response, log_errors, append = FALSE) {
 	}
 	if(500 <= code && code < 600) {
 		if(!is.null(log_errors)) {
-			stop(
+			warning(
 				"Your request generated a server error, HTTP code ",
 				code,
 				". For more information see the generated error file."
 			)
 		} else {
-			stop(
+			warning(
 				"Your request generated a server error, HTTP code ",
 				code,
 				". To print more detailed error information, run fhir_recent_http_error() or set argument log_errors to a filename and rerun you request."
