@@ -1,4 +1,4 @@
-## ---- include = FALSE---------------------------------------------------------
+## ----include = FALSE----------------------------------------------------------
 knitr::opts_chunk$set(
   collapse = TRUE,
   comment = "#"
@@ -108,6 +108,7 @@ head(table)
 fhir_canonical_design()
 
 ## -----------------------------------------------------------------------------
+#all attributes defined explicitly
 meds <- fhir_table_description(
 	resource = "MedicationStatement",
 	cols     = list(
@@ -115,11 +116,7 @@ meds <- fhir_table_description(
 		status_text = "text/status",
 		status      = "status",
 		med_system  = "medicationCodeableConcept/coding/system",
-		med_code    = "medicationCodeableConcept/coding/code",
-		med_display = "medicationCodeableConcept/coding/display",
-		dosage      = "dosage/text",
-		patient     = "subject/reference",
-		last_update = "meta/lastUpdated"
+		med_code    = "medicationCodeableConcept/coding/code"
 	),
 	sep           = "|",
 	brackets      = NULL,
@@ -128,8 +125,10 @@ meds <- fhir_table_description(
 	keep_attr     = FALSE 
 )
 
+#automatic extraction/default values
 pat <- fhir_table_description(resource = "Patient")
 
+#combine both table_descriptions
 design <- fhir_design(meds, pat)
 
 ## -----------------------------------------------------------------------------
@@ -186,6 +185,33 @@ df3 <- fhir_crack(bundles = bundle, design = desc2, format = "wide", verbose = 0
 
 df3
 
+## -----------------------------------------------------------------------------
+desc3 <- fhir_table_description(
+	resource = "Patient",
+	cols = c(id = "id",
+			 name = "name/given",
+			 address.city = "address[type[@value='physical'] and use[@value='home']]/city",
+			 address.country = "address[type[@value='physical'] and use[@value='home']]/country"
+			 )
+)
+
+df_selected <- fhir_crack(bundles = bundle, design = desc3, verbose = 0)
+df_selected
+
+## -----------------------------------------------------------------------------
+bundle2 <- fhir_unserialize(bundles = example_bundles5)
+desc4 <- fhir_table_description(resource = "Observation",
+								cols = c(
+									id = "id",
+									code = "code/coding[system[@value='http://loinc.org']]/code",
+								 	display = "code/coding[system[@value='http://loinc.org']]/display")
+									 )
+df_selected2 <- fhir_crack(bundles = bundle2,
+					design = desc4,
+					verbose = F)
+
+df_selected2
+
 ## -----------------------------------------------------------------------------
  fhir_melt(
 	indexed_data_frame = df2,
@@ -255,9 +281,49 @@ molten_2
 ## -----------------------------------------------------------------------------
 fhir_rm_indices(indexed_data_frame = molten_2, brackets = c("[", "]"))
 
+## -----------------------------------------------------------------------------
+fhir_melt_all(indexed_data_frame = df2, brackets = c("[", "]"), sep = " | ")
+
 ## -----------------------------------------------------------------------------
 fhir_cast(df2, brackets = c("[", "]"), sep = " | ", verbose = 0)
 
-## ---- include=F---------------------------------------------------------------
+## ----include=F----------------------------------------------------------------
 file.remove(paste0(temp_dir, "/design.xml"))
 
+## -----------------------------------------------------------------------------
+#unserialize example
+bundles <- fhir_unserialize(bundles = example_bundles7)
+
+#Define sep and brackets
+sep <- "|"
+brackets <- c("[", "]")
+
+#crack fhir resources
+table_desc <- fhir_table_description(
+    resource = "Patient",
+    brackets = brackets,
+    sep = sep
+)
+
+df <- fhir_crack(bundles = bundles, design = table_desc, verbose = 0)
+df
+
+## -----------------------------------------------------------------------------
+#name.given elements from the same name (i.e. the official vs. the nickname) 
+#should be collapsed
+
+df2 <- fhir_collapse(df, columns = "name.given", sep = sep, brackets = brackets)
+df2
+
+## -----------------------------------------------------------------------------
+df2_molten <- fhir_melt(indexed_data_frame =  df2, 
+						brackets = brackets, 
+						sep = sep, 
+						columns = fhir_common_columns(df2,"name"),
+						all_columns = TRUE
+						)
+df2_molten
+
+## -----------------------------------------------------------------------------
+fhir_rm_indices(df2_molten, brackets = brackets)
+
